from ..models import Emitter


class EmitterArgs():
  def __init__(self, name: str, short_id: str, schema: any):
    self.name = name
    self.short_id = short_id
    self.schema = schema

class EmitterProxy():
  def __init__(self, target_proxy, args: EmitterArgs, client):
    self.target = target_proxy
    self.args = args
    self.client = client

  async def save(self):
    emitter = Emitter(
      id=self.id(),
      name=self.args.name,
      schema=self.args.schema,
      target_id=self.target.id(),
      service_id=self.target.instance.args.service_id,
    )
    if emitter.id not in self.client.emitters or self.client.emitters[emitter.id].hash != emitter.hash:
      await self.client.set_data(emitter, 'Emitter')

  def id(self) -> str:
    return f"{self.target.instance.args.service_id}:{self.target.args.short_id}:{self.args.short_id}"
  
  async def pulse(self, data: any):
    await self.client.pulse_emitter(
      self.target.instance.args.service_id, 
      self.target.args.short_id,
      self.args.short_id,
      data,
    )


from datetime import datetime

from ..models import Target, TargetStatus, TargetStatusEnum
from .emitter import EmitterArgs, EmitterProxy
from .action import ActionArgs, ActionProxy

class TargetArgs():
    def __init__(self, name: str, short_id: str, category: str):
        self.name = name
        self.short_id = short_id
        self.category = category

class TargetProxy():
    def __init__(self, instance, args: TargetArgs, client):
        self.instance = instance
        self.args = args
        self.client = client

    async def add_emitter(self, args: EmitterArgs):
        emitter_proxy = EmitterProxy(self, args, self.client)
        await emitter_proxy.save()
        return emitter_proxy

    async def add_action(self, args: ActionArgs):
        action_proxy = ActionProxy(self, args, self.client)
        await action_proxy.save()
        return action_proxy
    # async def add_action(self, args: EmitterArgs):
    #    return

    async def save(self, status: TargetStatusEnum):
        target = Target(
          id=self.id(),
          name=self.args.name,
          category=self.args.category,
          bg_color=self.instance.args.color,
          fg_color=self.instance.args.color,
          parent_targets=[],
          sub_targets=[],
          service_id=self.instance.args.service_id,
          last_updated=datetime.now().isoformat(),
          root_level=True,
          # hash=generate_hash(),
        )

        status = TargetStatus(
            id=self.id(),
            name=self.args.name,
            target_id=self.id(),
            status=status,
            last_updated=datetime.now().isoformat(),
            instance_id=self.instance.id(),
            # hash=generate_hash()
        )

        if target.id not in self.client.targets or self.client.targets[target.id].hash != target.hash:
            await self.client.set_data(target, 'Target')

        if status.id not in self.client.target_statuses or self.client.target_statuses[status.id].hash != status.hash:
            await self.client.set_data(status, 'TargetStatus')

    def id(self) -> str:
        return f"{self.instance.args.service_id}:{self.args.short_id}"
    
    async def set_status(self, status: TargetStatusEnum):
        await self.save(status)


from .target import TargetProxy, TargetArgs
from .event_track import EventTrackProxy, EventTrackArgs
from ..models import TargetStatusEnum, Instance, InstanceStatusEnum, Machine

class InstanceArgs():
    def __init__(self, name: str, code: str, service_id: str, cluster_id: str, 
                 color: str, machine_id: str, message: str):
        self.name = name
        self.code = code
        self.service_id = service_id
        self.cluster_id = cluster_id
        self.machine_id = machine_id
        self.color = color
        self.message = message

class InstanceProxy():
    def __init__(self, args: InstanceArgs, client):
        self.args = args
        self.client = client
        self.targets = {}
        self.event_tracks = {}
  
    async def add_target(self, args: TargetArgs):
        target = TargetProxy(self, args, self.client)
        await target.save(TargetStatusEnum.Online)

        self.targets[target.id()] = target
        return target
    
    async def add_event_track(self, args: EventTrackArgs):
        event_track = EventTrackProxy(self, args, self.client)
        await event_track.save()

        self.event_tracks[event_track.id()] = event_track
        return event_track

    async def save(self, status: InstanceStatusEnum):
        instance = Instance(
            id=self.id(),
            name=self.args.name,
            service_id=self.args.service_id,
            service_type_code=self.args.code,
            cluster_id=self.args.cluster_id,
            machine_id=self.args.machine_id,
            color=self.args.color,
            message=self.args.message,
            status=status,
        )

        machine = Machine(
            name=self.args.machine_id,
            # hash=generate_hash()
        )
        # if instance.id not in self.client.instances or self.client.instances[instance.id].hash != instance.hash:
        await self.client.set_data(instance, 'Instance')

        # if machine.id not in self.client.machines or self.client.machines[machine.id].hash != machine.hash:
        await self.client.set_data(machine, 'Machine')

        for target in self.targets.values():
            await target.save(TargetStatusEnum.Online)

        print("Targets saved")
      
    async def set_status(self, status: InstanceStatusEnum):
        await self.save(status)

    def id(self) -> str:
        return f"{self.args.service_id}:{self.args.machine_id}"
    


from ..models import Action
from typing import Coroutine

class ActionArgs():
  def __init__(self, name: str, short_id: str, schema: any, handler: Coroutine[Action, any, None]):
    self.name = name
    self.short_id = short_id
    self.schema = schema
    self.handler = handler

class ActionProxy():
  def __init__(self, target_proxy, args: ActionArgs, client):
    self.target = target_proxy
    self.args = args
    self.client = client
  

  async def save(self):
    action = Action(
      id=self.id(),
      name=self.args.name,
      schema=self.args.schema,
      target_id=self.target.id(),
      service_id=self.target.instance.args.service_id,
    )

    
    if action.id not in self.client.actions or self.client.actions[action.id].hash != action.hash:
      await self.client.set_data(action, 'Action')
      self.client.actions[action.id] = action
      self.client.handlers[action.id] = self.args.handler


  def id(self) -> str:
    return f"{self.target.instance.args.service_id}:{self.target.args.short_id}:{self.args.short_id}"
